#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file rtc_3DImageShapeGUI.py
 @brief Processing of three dimensional image shape system GUI.
 @date $Date$

 @author EAST JAPAN ACCOUNTING CENTER CO.,LTD.

 BSD 3-clause "New" or "Revised" License

"""
import sys
# import time

# Import RTM module
import RTC
import OpenRTM_aist

# add to code
import os
import os.path
import unicodedata
import copy
import datetime
import dateutil.parser
import shutil

from PyQt4 import QtCore, QtGui
from gui_3d_image_shape import Ui_MainWindow
from sub_file_selection import Ui_Dialog

sys.path.append(".")

# add to code
""" message definition """
messege_text_01 = ("onInitialize")
messege_text_02 = ("onActivated")
messege_text_03 = ("onDeactivated")
messege_text_04 = ("Heart beat by 3D image shape GUI")
messege_text_05 = (
    "Error was detected. \
Please reset the 3D image shape GUI RTC by the RT System Editor.")

messege_error_01 = (
    "Please press the switch after activation of the RTC.")
messege_error_02 = (
    "ERROR: Japanese characters does not supported!")
messege_error_03 = ("File does not exist.")
messege_error_04 = (
    "ERROR: File name length is up to 45 characters. Please reconfirm.")
messege_error_05 = ("Please file extension ftv.")
messege_error_06 = ("Please select a different file.")

""" constant definitions """
WINDOW_M_SIZE_W = (400)
WINDOW_M_SIZE_H = (720)

WINDOW_S_SIZE_W = (720)
WINDOW_S_SIZE_H = (720)

NUM_OF_CHAR_MAX = (46)

WINDOW_TAB__CAM = (0)
WINDOW_TAB__CAP = (1)
WINDOW_TAB_CLD1 = (2)
WINDOW_TAB_CLD2 = (3)
WINDOW_TAB__STD = (4)

KIND_F_PATH_LFT = (0)
KIND_F_PATH_CEN = (1)
KIND_F_PATH_RHT = (2)

KIND_F_NAME_LFT = (0)
KIND_F_NAME_CEN = (1)
KIND_F_NAME_RHT = (2)

DF_OF_ACTIVATED = (0)
DF_ON_ACTIVATED = (1)

DF_DEFAULT_0SET = (0)
DF_DEFAULT_1SET = (1)
DF_DEFAULT_1CHR = ("1")

DEFINE_MODE_CAM = ("camera")
DEFINE_MODE_CAP = ("capture")
DEFINE_MODE_CLD = ("cloud")
DEFINE_MODE_STD = ("storeddata")

DEFINE_MODE_PLY = ("play")
DEFINE_MODE_STP = ("stop")

W_FRAME_MOD__ON = ("wire_frame_on")
W_FRAME_MOD_OFF = ("wire_frame_of")

LIST_LENGTH_MAX = (500)

DB_CNECT_CHR_NG = ("0")
DB_CNECT_CHR_OK = ("1")

DB_CONNECT_INIT = (0)
DB_CNECT_NUM_NG = DB_CONNECT_INIT
DB_CNECT_NUM_OK = (1)

DEFINE_KIND_FTV = (0)
DEFINE_KIND_BMP = (1)

DATE_TIME_INIT0 = (0)
DEFINE_LENGTH_0 = (0)

DF_EMPTY_STRING = ""

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
rtc_3dimageshapegui_spec = ["implementation_id", "rtc_3DImageShapeGUI",
                            "type_name",         "rtc_3DImageShapeGUI",
                            "description",       "Processing \
                               of three dimensional image shape system GUI.",
                            "version",           "1.0",
                            "vendor",            "EAC",
                            "category",          "Image",
                            "activity_type",     "STATIC",
                            "max_instance",      "1",
                            "language",          "Python",
                            "lang_type",         "SCRIPT",
                            ""]
# </rtc-template>

##
# @class rtc_3DImageShapeGUI
# @brief Processing of three dimensional image shape system GUI.
#
# Processing of three dimensional image shape system GUI
#
# 3D image object path information and the file-name-list
#    of MongoDB search results
#
# -
#
# -
#
#


class rtc_3DImageShapeGUI(OpenRTM_aist.DataFlowComponentBase):

    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_in_filelist = RTC.TimedStringSeq(RTC.Time(0, 0), [])
        """
        This port inputs the file-name-list of search results.
         - Type: RTC::TimedStringSeq
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._file_list_inIn = OpenRTM_aist.InPort(
            "file_list_in", self._d_in_filelist)
        self._d_connection_sts = RTC.TimedString(RTC.Time(0, 0), "")
        """
        This port inputs the MongoDB connection status.
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._DBConnectionStatusIn = OpenRTM_aist.InPort(
            "DBConnectionStatus", self._d_connection_sts)
        self._d_in_filepath_3d = RTC.TimedString(RTC.Time(0, 0), "")
        """
        This port inputs the path information of the 3D shape object.
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._file_path_3dobjIn = OpenRTM_aist.InPort(
            "file_path_3dobj", self._d_in_filepath_3d)
        self._d_out_condition = RTC.TimedStringSeq(RTC.Time(0, 0), [])
        """
        This port outputs file-name-list the search condition.
         - Type: RTC::TimedStringSeq
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._db_search_conditionOut = OpenRTM_aist.OutPort(
            "db_search_condition", self._d_out_condition)
        self._d_out_file_name_m = RTC.TimedStringSeq(RTC.Time(0, 0), [])
        """
        This port outputs the path information acquisition request of multiple
        image files.
         - Type: RTC::TimedStringSeq
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._file_search_multiOut = OpenRTM_aist.OutPort(
            "file_search_multi", self._d_out_file_name_m)
        self._d_out_file_name_s = RTC.TimedString(RTC.Time(0, 0), "")
        """
        This port outputs the path information acquisition request
        of single image files.
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._file_search_singleOut = OpenRTM_aist.OutPort(
            "file_search_single", self._d_out_file_name_s)
        self._d_out_filepath_m = RTC.TimedStringSeq(RTC.Time(0, 0), [])
        """
        This port outputs the multiple file path information.
         - Type: RTC::TimedStringSeq
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._file_path_multiOut = OpenRTM_aist.OutPort(
            "file_path_multi", self._d_out_filepath_m)
        self._d_out_filepath_s = RTC.TimedString(RTC.Time(0, 0), "")
        """
        This port outputs the single file path information.
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._file_path_singleOut = OpenRTM_aist.OutPort(
            "file_path_single", self._d_out_filepath_s)
        self._d_sys_mode = RTC.TimedString(RTC.Time(0, 0), "")
        """
        This port outputs the mode information of the 3-D images-shape-system.
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._ModeOut_3DSystemOut = OpenRTM_aist.OutPort(
            "ModeOut_3DSystem", self._d_sys_mode)
        self._d_play_mode = RTC.TimedString(RTC.Time(0, 0), "")
        """
        This port outputs a start-information and stop-information of the 3-D
        images-shape-system.
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._StartOut_3DSystemOut = OpenRTM_aist.OutPort(
            "StartOut_3DSystem", self._d_play_mode)
        self._d_out_opngl_path = RTC.TimedString(RTC.Time(0, 0), "")
        """
        This port outputs the path information of the opengl file.
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._opengl_file_pathOut = OpenRTM_aist.OutPort(
            "opengl_file_path", self._d_out_opngl_path)
        self._d_out_w_frame = RTC.TimedString(RTC.Time(0, 0), "")
        """
        This port outputs the wire frame mode information.
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._out_wire_frameOut = OpenRTM_aist.OutPort(
            "out_wire_frame", self._d_out_w_frame)
        self._d_out_z_slider_val = RTC.TimedString(RTC.Time(0, 0), "")
        """
        This port outputs the value of Z coordinate.
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: two cycles per second.
        """
        self._out_z_sliderOut = OpenRTM_aist.OutPort(
            "out_z_slider", self._d_out_z_slider_val)

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">

        # </rtc-template>

    ##
    # Initialization of the 3D image shape GUI RTC
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    # @pre -
    # @post -
    #
    def onInitialize(self):
        # Bind variables and configuration variable

        # Set InPort buffers
        self.addInPort("file_list_in", self._file_list_inIn)
        self.addInPort("DBConnectionStatus", self._DBConnectionStatusIn)
        self.addInPort("file_path_3dobj", self._file_path_3dobjIn)

        # Set OutPort buffers
        self.addOutPort("db_search_condition", self._db_search_conditionOut)
        self.addOutPort("file_search_multi", self._file_search_multiOut)
        self.addOutPort("file_search_single", self._file_search_singleOut)
        self.addOutPort("file_path_multi", self._file_path_multiOut)
        self.addOutPort("file_path_single", self._file_path_singleOut)
        self.addOutPort("ModeOut_3DSystem", self._ModeOut_3DSystemOut)
        self.addOutPort("StartOut_3DSystem", self._StartOut_3DSystemOut)
        self.addOutPort("opengl_file_path", self._opengl_file_pathOut)
        self.addOutPort("out_wire_frame", self._out_wire_frameOut)
        self.addOutPort("out_z_slider", self._out_z_sliderOut)

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        # add to code
        self.onInitializeFunc()
        print messege_text_01

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The finalize action (on ALIVE->END transition)
    #    # formaer rtc_exiting_entry()
    #    #
    #    # @return RTC::ReturnCode_t
    #
    #    #
    # def onFinalize(self):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The startup action when ExecutionContext startup
    #    # former rtc_starting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The shutdown action when ExecutionContext stop
    #    # former rtc_stopping_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK

        ##
        # Activate the 3D image shape GUI RTC
        #
        # The activated action (Active state entry action)
        # former rtc_active_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onActivated(self, ec_id):

        # add to code
        self.onActivatedFunc()
        print messege_text_02

        return RTC.RTC_OK

        ##
        # Deactivate the 3D image shape GUI RTC
        #
        # The deactivated action (Active state exit action)
        # former rtc_active_exit()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onDeactivated(self, ec_id):

        # add to code
        self.onDeactivatedFunc()
        print messege_text_03

        return RTC.RTC_OK

        ##
        # Periodic task of the 3D image shape GUI RTC
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onExecute(self, ec_id):

        # add to code
        self.onExecute_Check_input_of_DataPort()
        # print messege_text_04

        return RTC.RTC_OK

        ##
        # onAborting the 3D image shape GUI RTC
        #
        # The aborting action when main logic error occurred.
        # former rtc_aborting_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onAborting(self, ec_id):

        # add to code
        print messege_text_05

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The error action in ERROR state
    #    # former rtc_error_do()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onError(self, ec_id):
    #
    #    return RTC.RTC_OK

        ##
        # Reset initialization of the 3D image shape GUI RTC
        #
        # The reset action that is invoked resetting
        # This is same but different the former rtc_init_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onReset(self, ec_id):

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The state update action that is invoked after onExecute() action
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #

    #    #
    # def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The action that is invoked when execution context's rate is changed
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    # =============================================================================================
    # Function Name   :onExecute_Check_input_of_DataPort
    # Description     :This function checks the input data port.
    # Argument        :inputs data-port
    # Return          :nothing special
    # =============================================================================================
    def onExecute_Check_input_of_DataPort(self):
        if self._file_list_inIn.isNew():
            file_name_list = self._file_list_inIn.read()
            self.set_file_name_list(file_name_list)

        if self._file_path_3dobjIn.isNew():
            file_path_3dobj = self._file_path_3dobjIn.read()
            if self.check_of_play_mode(DEFINE_MODE_PLY) is True:
                if self.check_of_sys_mode(DEFINE_MODE_STD) is False:
                    self.set_file_path_of_3d_object(file_path_3dobj)

        if self._DBConnectionStatusIn.isNew():
            db_connection_status = self._DBConnectionStatusIn.read()
            self.set_db_connection_status(db_connection_status)

    # =============================================================================================
    # Function Name   :set_file_path_of_3d_object
    # Description     :This function sets the input 3D object file path
    #                 :to internal variables.
    # Argument        :file name list of image
    # Return          :nothing special
    # =============================================================================================
    def set_file_path_of_3d_object(self, file_path_val):
        if self.check_3d_obj_file_exists(file_path_val):
            self.create_3d_shape_dir()
            self.copy_3d_shape_file(DEFINE_KIND_FTV)
            self.copy_3d_shape_file(DEFINE_KIND_BMP)

            self.outputs_viewer_related_data()

    # =============================================================================================
    # Function Name   :check_3d_obj_file_exists
    # Description     :This function confirms the existence of 3D obj file.
    # Argument        :file name list of image
    # Return          :information of normal end(True) or abnormal end(False).
    # =============================================================================================
    def check_3d_obj_file_exists(self, path_val):
        ret_val = False

        original_path = path_val.data
        src_3dobj_dir = original_path.replace(os.path.sep, '/')
        path_ftv = src_3dobj_dir + "/3d_shape.ftv"
        path_bmp = src_3dobj_dir + "/3d_shape.bmp"

        if self.g_check_file_exists(path_ftv) is True:
            if self.g_check_file_exists(path_bmp) is True:
                self.file_path_ftv_src = copy.deepcopy(path_ftv)
                self.file_path_bmp_src = copy.deepcopy(path_bmp)
                ret_val = True

        return ret_val

    # =============================================================================================
    # Function Name   :create_3d_shape_dir
    # Description     :This function creates a directory of 3d shape.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def create_3d_shape_dir(self):
        if os.path.isdir(self.get_absolute_3d_shape_dir()) is False:
            os.mkdir("./3d_shape")

    # =============================================================================================
    # Function Name   :create_stored_data_dir
    # Description     :This function creates a directory of stored data.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def create_stored_data_dir(self):
        absolute_dir = self.absolute_dir_path + "/stored_data"

        if os.path.isdir(absolute_dir) is False:
            os.mkdir("./stored_data")

    # =============================================================================================
    # Function Name   :copy_3d_shape_file
    # Description     :This function copies 3d object file and bmp file.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def copy_3d_shape_file(self, kind):
        if kind == DEFINE_KIND_FTV:
            dst_file = self.get_absolute_3d_file_path(DEFINE_KIND_FTV)
            src_file = self.file_path_ftv_src
            self.file_path_ftv_dst = copy.deepcopy(dst_file)
        else:
            dst_file = self.get_absolute_3d_file_path(DEFINE_KIND_BMP)
            src_file = self.file_path_bmp_src
            self.file_path_bmp_dst = copy.deepcopy(dst_file)

        if os.path.exists(dst_file) is True:
            os.remove(dst_file)
        shutil.copyfile(src_file, dst_file)

    # =============================================================================================
    # Function Name   :del_3d_shape_file
    # Description     :This function deletes 3d object file and bmp file.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def del_3d_shape_file(self):
        f_path_ftv = self.get_absolute_3d_file_path(DEFINE_KIND_FTV)
        f_path_bmp = self.get_absolute_3d_file_path(DEFINE_KIND_BMP)

        if os.path.exists(f_path_ftv) is True:
            os.remove(f_path_ftv)

        if os.path.exists(f_path_bmp) is True:
            os.remove(f_path_bmp)

        self.file_path_ftv_dst = DF_EMPTY_STRING
        self.file_path_bmp_dst = DF_EMPTY_STRING

    # =============================================================================================
    # Function Name   :set_file_name_list
    # Description     :This function sets the input file-name-list
    #                 :to internal variables.
    # Argument        :file name list of image
    # Return          :nothing special
    # =============================================================================================
    def set_file_name_list(self, name_list):
        list_len = len(name_list.data)
        if list_len > DEFINE_LENGTH_0:
            if list_len <= LIST_LENGTH_MAX:

                del self.name_list[:]
                for data in name_list.data:
                    self.name_list.append(data)

            else:
                QtGui.QMessageBox.warning(
                    self, "Message",
                    u"The number of input file name lists is large. \
Please reconsider the search condition of the database.")

        else:
            QtGui.QMessageBox.warning(
                self, "Message",
                u"The number of entered file name lists is wrong.")

    # =============================================================================================
    # Function Name   :set_db_connection_status
    # Description     :This function sets the input DB connection status
    #                 :to internal variables.
    # Argument        :file name list of image
    # Return          :nothing special
    # =============================================================================================
    def set_db_connection_status(self, db_connection_sts):
        result = True
        if db_connection_sts.data == DB_CNECT_CHR_NG:
            self.db_connect_sts = DB_CNECT_NUM_NG

        elif db_connection_sts.data == DB_CNECT_CHR_OK:
            self.db_connect_sts = DB_CNECT_NUM_OK

        else:
            self.db_connect_sts = DB_CNECT_NUM_NG
            result = False

        if result is True:
            connection_status = copy.deepcopy(int(db_connection_sts.data))
            print ("connection status = %d" % connection_status)

    # Definition of function
    # Main window function
    # =============================================================================================
    # Function Name   :gui_camera_func
    # Description     :The gui_camera_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def gui_camera_func(self):
        self.sys_mode = DEFINE_MODE_CAM

        print("camera button")

        self.outputs_sys_mode_func(DEFINE_MODE_CAM)

    # =============================================================================================
    # Function Name   :gui_capture_func
    # Description     :The gui_capture_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def gui_capture_func(self):
        self.sys_mode = DEFINE_MODE_CAP

        print("capture button")

        self.outputs_sys_mode_func(DEFINE_MODE_CAP)

    # =============================================================================================
    # Function Name   :gui_cloud_func
    # Description     :The gui_cloud_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def gui_cloud_func(self):
        self.sys_mode = DEFINE_MODE_CLD

        print("cloud button")

        self.outputs_sys_mode_func(DEFINE_MODE_CLD)

    # =============================================================================================
    # Function Name   :gui_stored_data_func
    # Description     :The gui_stored_data_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def gui_stored_data_func(self):
        self.sys_mode = DEFINE_MODE_STD

        print("stored data button")

        self.outputs_sys_mode_func(DEFINE_MODE_STD)

    # =============================================================================================
    # Function Name   :outputs_sys_mode_func
    # Description     :This function outputs system mode.
    # Argument        :system mode
    # Return          :nothing special
    # =============================================================================================
    def outputs_sys_mode_func(self, mode):
        self._d_sys_mode.tm.sec = copy.deepcopy(DATE_TIME_INIT0)
        self._d_sys_mode.tm.nsec = copy.deepcopy(DATE_TIME_INIT0)

        self._d_sys_mode.data = copy.deepcopy(mode)

        ret = self._ModeOut_3DSystemOut.write()
        if not ret:
            print "!write error"

    # =============================================================================================
    # Function Name   :gui_stored_data_file_path_func
    # Description     :This function is executed
    #                 :     when a ftv-file is selected in the browser.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def gui_stored_data_file_path_func(self, path_ftv):
        dir_name = os.path.dirname(path_ftv)
        file_name = os.path.basename(path_ftv)
        f_name, ext = os.path.splitext(file_name)
        path_bmp = (dir_name + "/" + f_name + ".bmp")

        if self.g_check_file_exists(path_bmp) is True:
            # str -> unicode
            src_path_ftv = self.change_str_unicode(path_ftv)
            src_path_bmp = self.change_str_unicode(path_bmp)

            dst_ftv = self.get_absolute_3d_file_path(DEFINE_KIND_FTV)
            dst_bmp = self.get_absolute_3d_file_path(DEFINE_KIND_BMP)
            # str -> unicode
            dst_path_ftv = self.change_str_unicode(dst_ftv)
            dst_path_bmp = self.change_str_unicode(dst_bmp)

            if src_path_ftv != dst_path_ftv:
                if os.path.exists(dst_path_ftv) is True:
                    os.remove(dst_path_ftv)
                if os.path.exists(dst_path_bmp) is True:
                    os.remove(dst_path_bmp)

                shutil.copyfile(src_path_ftv, dst_path_ftv)
                shutil.copyfile(src_path_bmp, dst_path_bmp)
                self.outputs_viewer_related_data()

            else:
                print messege_error_06

        else:
            print (messege_error_03 + " file name: " + f_name + ".bmp")

    # =============================================================================================
    # Function Name   :outputs_viewer_related_data
    # Description     :This function outputs the Viewer related data.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def outputs_viewer_related_data(self):

        self.outputs_wire_frame_mode(self.wire_frame_mode)
        self.outputs_z_coordinate_val(self.slider_value)
        self.outputs_3d_object_file()

    # =============================================================================================
    # Function Name   :gui_stored_data_file_name_func
    # Description     :This function is executed when the file is set
    #                 :     by the browser. And this function copies the file
    #                 :     to the specified directory.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def gui_stored_data_file_name_func(self, file_name, file_path):

        dst_ftv = file_path + "/" + file_name + ".ftv"
        dst_bmp = file_path + "/" + file_name + ".bmp"
        dst_path_ftv = self.change_str_unicode(dst_ftv)
        dst_path_bmp = self.change_str_unicode(dst_bmp)

        src_ftv = self.get_absolute_3d_file_path(DEFINE_KIND_FTV)
        src_bmp = self.get_absolute_3d_file_path(DEFINE_KIND_BMP)
        src_path_ftv = self.change_str_unicode(src_ftv)
        src_path_bmp = self.change_str_unicode(src_bmp)

        result = False
        if os.path.exists(src_path_ftv) is True:
            if os.path.exists(src_path_bmp) is True:
                result = True

        if result is True:
            if os.path.exists(dst_path_ftv) is True:
                os.remove(dst_path_ftv)
            if os.path.exists(dst_path_bmp) is True:
                os.remove(dst_path_bmp)

            shutil.copyfile(src_path_ftv, dst_path_ftv)
            shutil.copyfile(src_path_bmp, dst_path_bmp)

    # =============================================================================================
    # Function Name   :gui_play_mode_func
    # Description     :This function is executed
    #                 :when the "play" and "stop" radio buttons are pressed.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def gui_play_mode_func(self, play_mode):
        result = True
        if play_mode == DEFINE_MODE_PLY:
            self.ply_mode = DEFINE_MODE_PLY
        elif play_mode == DEFINE_MODE_STP:
            self.ply_mode = DEFINE_MODE_STP
        else:
            result = False

        if result is True:
            self._d_play_mode.tm.sec = copy.deepcopy(DATE_TIME_INIT0)
            self._d_play_mode.tm.nsec = copy.deepcopy(DATE_TIME_INIT0)

            self._d_play_mode.data = copy.deepcopy(
                self.change_unicode_str(play_mode))

            ret = self._StartOut_3DSystemOut.write()
            if not ret:
                print "!write error"

    # =============================================================================================
    # Function Name   :outputs_3d_object_file
    # Description     :This function outputs information
    #                 :on the 3D object file path.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def outputs_3d_object_file(self):
        absolute_dir = self.get_absolute_3d_shape_dir()

        result = False
        if os.path.isdir(absolute_dir) is True:
            f_path_ftv = self.get_absolute_3d_file_path(DEFINE_KIND_FTV)
            f_path_bmp = self.get_absolute_3d_file_path(DEFINE_KIND_BMP)
            if os.path.exists(f_path_ftv) is True:
                if os.path.exists(f_path_bmp) is True:
                    result = True

        if result is True:
            self._d_out_opngl_path.tm.sec = copy.deepcopy(DATE_TIME_INIT0)
            self._d_out_opngl_path.tm.nsec = copy.deepcopy(DATE_TIME_INIT0)

            self._d_out_opngl_path.data = copy.deepcopy(
                self.change_unicode_str(absolute_dir))

            ret = self._opengl_file_pathOut.write()
            if not ret:
                print "!write error"

    # =============================================================================================
    # Function Name   :gui_wire_frame_mode_func
    # Description     :This function is executed
    #                 :    when the "wire frame off" and "wire frame on"
    #                 :        radio buttons are pressed.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def gui_wire_frame_mode_func(self, wire_frame):
        result = True
        if wire_frame == W_FRAME_MOD__ON:
            pass
        elif wire_frame == W_FRAME_MOD_OFF:
            pass
        else:
            result = False

        if result is True:
            self.wire_frame_mode = copy.deepcopy(wire_frame)
            self.outputs_wire_frame_mode(wire_frame)

    # =============================================================================================
    # Function Name   :outputs_wire_frame_mode
    # Description     :This function outputs information
    #                 :on the wire frame mode.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def outputs_wire_frame_mode(self, wire_frame):

        if len(wire_frame) != DEFINE_LENGTH_0:

            self._d_out_w_frame.tm.sec = copy.deepcopy(DATE_TIME_INIT0)
            self._d_out_w_frame.tm.nsec = copy.deepcopy(DATE_TIME_INIT0)

            self._d_out_w_frame.data = copy.deepcopy(
                self.change_unicode_str(wire_frame))
            ret = self._out_wire_frameOut.write()
            if not ret:
                print "!write error"

    # =============================================================================================
    # Function Name   :gui_z_coordinate_slider_func
    # Description     :This function gets the value of the slider.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def gui_z_coordinate_slider_func(self, slider_val):
        self.slider_value = copy.deepcopy(slider_val)
        self.outputs_z_coordinate_val(slider_val)

    # =============================================================================================
    # Function Name   :outputs_z_coordinate_val
    # Description     :This function outputs information
    #                 :on value of the z coordinate slider.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def outputs_z_coordinate_val(self, slider_value):

        if len(slider_value) != DEFINE_LENGTH_0:

            self._d_out_z_slider_val.tm.sec = copy.deepcopy(DATE_TIME_INIT0)
            self._d_out_z_slider_val.tm.nsec = copy.deepcopy(DATE_TIME_INIT0)

            self._d_out_z_slider_val.data = copy.deepcopy(
                self.change_unicode_str(slider_value))

            ret = self._out_z_sliderOut.write()
            if not ret:
                print "!write error"

    # =============================================================================================
    # Function Name   :gui_set_file_path
    # Description     :This function
    #                 :sets the file path information of the image.
    # Argument        :image path information
    # Return          :nothing special
    # =============================================================================================
    def gui_set_file_path(self, kind_val, file_path):
        if kind_val == KIND_F_PATH_LFT:
            self.file_path_lft = file_path

        elif kind_val == KIND_F_PATH_CEN:
            self.file_path_cen = file_path

        elif kind_val == KIND_F_PATH_RHT:
            self.file_path_rht = file_path

        else:
            pass

    # =============================================================================================
    # Function Name   :gui_chk_image
    # Description     :This function
    #                 :outputs the file path information of the image.
    # Argument        :image path information
    # Return          :nothing special
    # =============================================================================================
    def gui_chk_image(self, file_path):
        if len(file_path) != DEFINE_LENGTH_0:
            self._d_out_filepath_s.tm.sec = copy.deepcopy(DATE_TIME_INIT0)
            self._d_out_filepath_s.tm.nsec = copy.deepcopy(DATE_TIME_INIT0)

            self._d_out_filepath_s.data = copy.deepcopy(
                self.change_unicode_str(file_path))

            ret = self._file_path_singleOut.write()
            if not ret:
                print "!write error"

    # =============================================================================================
    # Function Name   :gui_select_complete_func
    # Description     :This function
    #                 :outputs multiple file path information to the port.
    # Argument        :image path information
    # Return          :nothing special
    # =============================================================================================
    def gui_select_complete_func(self):
        result = False
        if len(self.file_path_lft) != DEFINE_LENGTH_0:
            f_path_lft = self.change_unicode_str(self.file_path_lft)
            if len(self.file_path_cen) != DEFINE_LENGTH_0:
                f_path_cen = self.change_unicode_str(self.file_path_cen)
                if len(self.file_path_rht) != DEFINE_LENGTH_0:
                    f_path_rht = self.change_unicode_str(self.file_path_rht)
                    result = True

        if result is True:
            self._d_out_filepath_m.tm.sec = copy.deepcopy(DATE_TIME_INIT0)
            self._d_out_filepath_m.tm.nsec = copy.deepcopy(DATE_TIME_INIT0)

            file_path_list = []
            file_path_list.append(f_path_lft)
            file_path_list.append(f_path_cen)
            file_path_list.append(f_path_rht)

            del self._d_out_filepath_m.data[:]
            self._d_out_filepath_m.data = copy.deepcopy(file_path_list)

            ret = self._file_path_multiOut.write()
            if not ret:
                print "!write error"

    # =============================================================================================
    # Function Name   :gui_path_inf_clear_func
    # Description     :This function deletes the file path information.
    # Argument        :image path information
    # Return          :nothing special
    # =============================================================================================
    def gui_path_inf_clear_func(self):
        if len(self.file_path_lft) != DEFINE_LENGTH_0:
            self.file_path_lft = DF_EMPTY_STRING

        if len(self.file_path_cen) != DEFINE_LENGTH_0:
            self.file_path_cen = DF_EMPTY_STRING

        if len(self.file_path_rht) != DEFINE_LENGTH_0:
            self.file_path_rht = DF_EMPTY_STRING

    # =============================================================================================
    # Function Name   :gui_chk_image_cloud
    # Description     :This function
    #                 :outputs the file name information of the image.
    # Argument        :image path information
    # Return          :nothing special
    # =============================================================================================
    def gui_chk_image_cloud(self, file_name):
        if len(file_name) != DEFINE_LENGTH_0:
            self._d_out_file_name_s.tm.sec = copy.deepcopy(DATE_TIME_INIT0)
            self._d_out_file_name_s.tm.nsec = copy.deepcopy(DATE_TIME_INIT0)

            self._d_out_file_name_s.data = copy.deepcopy(
                self.change_unicode_str(file_name))

            ret = self._file_search_singleOut.write()
            if not ret:
                print "!write error"

    # =============================================================================================
    # Function Name   :gui_set_file_name
    # Description     :This function
    #                 :sets the file name information of the image.
    # Argument        :image path information
    # Return          :nothing special
    # =============================================================================================
    def gui_set_file_name(self, file_name, kind):
        if kind == KIND_F_NAME_LFT:
            self.file_name_lft = file_name
        elif kind == KIND_F_NAME_CEN:
            self.file_name_cen = file_name
        elif kind == KIND_F_NAME_RHT:
            self.file_name_rht = file_name
        else:
            pass

    # =============================================================================================
    # Function Name   :gui_select_complete_cloud_func
    # Description     :This function
    #                 :outputs multiple file name information to the port.
    # Argument        :image path information
    # Return          :nothing special
    # =============================================================================================
    def gui_select_complete_cloud_func(self):
        result = False
        if len(self.file_name_lft) != DEFINE_LENGTH_0:
            f_name_lft = self.change_unicode_str(self.file_name_lft)
            if len(self.file_name_cen) != DEFINE_LENGTH_0:
                f_name_cen = self.change_unicode_str(self.file_name_cen)
                if len(self.file_name_rht) != DEFINE_LENGTH_0:
                    f_name_rht = self.change_unicode_str(self.file_name_rht)
                    result = True

        if result is True:
            self._d_out_file_name_m.tm.sec = copy.deepcopy(DATE_TIME_INIT0)
            self._d_out_file_name_m.tm.nsec = copy.deepcopy(DATE_TIME_INIT0)

            file_name_list = []
            file_name_list.append(f_name_lft)
            file_name_list.append(f_name_cen)
            file_name_list.append(f_name_rht)

            del self._d_out_file_name_m.data[:]
            self._d_out_file_name_m.data = copy.deepcopy(file_name_list)

            ret = self._file_search_multiOut.write()
            if not ret:
                print "!write error"

    # =============================================================================================
    # Function Name   :gui_file_inf_clear_func
    # Description     :This function deletes the file name information.
    # Argument        :image path information
    # Return          :nothing special
    # =============================================================================================
    def gui_file_inf_clear_func(self):
        if len(self.file_name_lft) != DEFINE_LENGTH_0:
            self.file_name_lft = DF_EMPTY_STRING

        if len(self.file_name_cen) != DEFINE_LENGTH_0:
            self.file_name_cen = DF_EMPTY_STRING

        if len(self.file_name_rht) != DEFINE_LENGTH_0:
            self.file_name_rht = DF_EMPTY_STRING

    # =============================================================================================
    # Function Name   :gui_outputs_db_search_query
    # Description     :This function outputs the search_query information.
    # Argument        :date information
    # Return          :nothing special
    # =============================================================================================
    def gui_outputs_db_search_query(
            self, robot_id, year_min, mon_min, day_min, year_max, mon_max,
            day_max, h_min, m_min, s_min, h_max, m_max, s_max):

        self._d_out_condition.tm.sec = copy.deepcopy(DATE_TIME_INIT0)
        self._d_out_condition.tm.nsec = copy.deepcopy(DATE_TIME_INIT0)

        search_query_val = []
        search_query_val.append(self.change_unicode_str(year_min))
        search_query_val.append(self.change_unicode_str(mon_min))
        search_query_val.append(self.change_unicode_str(day_min))
        search_query_val.append(self.change_unicode_str(h_min))
        search_query_val.append(self.change_unicode_str(m_min))
        search_query_val.append(self.change_unicode_str(s_min))
        search_query_val.append(self.change_unicode_str(year_max))
        search_query_val.append(self.change_unicode_str(mon_max))
        search_query_val.append(self.change_unicode_str(day_max))
        search_query_val.append(self.change_unicode_str(h_max))
        search_query_val.append(self.change_unicode_str(m_max))
        search_query_val.append(self.change_unicode_str(s_max))

        search_query_val.append(self.change_unicode_str(robot_id))

        del self._d_out_condition.data[:]
        self._d_out_condition.data = copy.deepcopy(search_query_val)

        print "DATE MIN: " + year_min + "/ " + mon_min + "/ " + day_min
        print "DATE MAX: " + year_max + "/ " + mon_max + "/ " + day_max
        print "TIME MIN: " + h_min + " : " + m_min + " : " + s_min
        print "TIME MAX: " + h_max + " : " + m_max + " : " + s_max

        ret = self._db_search_conditionOut.write()
        if not ret:
            print "!write error"

    # =============================================================================================
    # Function Name   :g_check_file_exists
    # Description     :This function checks whether a file exists.
    # Argument        :file path
    # Return          :information of normal end or abnormal end.
    # =============================================================================================
    def g_check_file_exists(self, value):
        if os.path.exists(value) is False:
            return False
        else:
            if os.path.isfile(value) is False:
                return False
        return True

    # =============================================================================================
    # Function Name   :onInitializeFunc
    # Description     :This function is an "onInitialize process"
    #                  run-time initialization process.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def onInitializeFunc(self):
        self.activated_sts = DF_OF_ACTIVATED
        self.sys_mode = DEFINE_MODE_CAM
        self.db_connect_sts = DB_CNECT_NUM_NG
        self.ply_mode = DEFINE_MODE_STP

        directory_path = os.path.abspath(os.path.dirname(__file__))
        self.absolute_dir_path = directory_path.replace(os.path.sep, '/')

    # =============================================================================================
    # Function Name   :onActivatedFunc
    # Description     :This function is an "onActivated process"
    #                  run-time initialization process.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def onActivatedFunc(self):
        self.activated_sts = DF_ON_ACTIVATED
        self.db_connect_sts = DB_CONNECT_INIT
        self.sys_mode = DEFINE_MODE_CAM
        self.ply_mode = DEFINE_MODE_STP

        self.file_path_lft = DF_EMPTY_STRING
        self.file_path_cen = DF_EMPTY_STRING
        self.file_path_rht = DF_EMPTY_STRING

        self.file_name_lft = DF_EMPTY_STRING
        self.file_name_cen = DF_EMPTY_STRING
        self.file_name_rht = DF_EMPTY_STRING

        self.wire_frame_mode = DF_EMPTY_STRING
        self.slider_value = DF_EMPTY_STRING

        self.file_path_ftv_src = DF_EMPTY_STRING
        self.file_path_bmp_src = DF_EMPTY_STRING

        self.del_3d_shape_file()

        self.name_list = []
        self._d_out_condition.data = []
        self._d_out_file_name_m.data = []
        self._d_out_filepath_m.data = []

        self.create_stored_data_dir()

        my_gui_form.global_gui_initialize()

    # =============================================================================================
    # Function Name   :onDeactivatedFunc
    # Description     :This function is an "onDeactivated process"
    #                  run-time initialization process.
    # Argument        :connection status inf
    # Return          :nothing special
    # =============================================================================================
    def onDeactivatedFunc(self):
        self.activated_sts = DF_OF_ACTIVATED

        list_len = len(self.name_list)
        if list_len > DEFINE_LENGTH_0:
            del self.name_list[:]

    # =============================================================================================
    # Function Name   :get_absolute_3d_shape_dir
    # Description     :This function gets path information
    #                 :of the 3d_shape directory.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def get_absolute_3d_shape_dir(self):
        absolute_dir = self.absolute_dir_path + "/3d_shape"

        return absolute_dir

    # =============================================================================================
    # Function Name   :get_absolute_3d_file_path
    # Description     :This function gets path information
    #                 :of ftv file and bmp file.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def get_absolute_3d_file_path(self, kind):
        if kind == DEFINE_KIND_FTV:
            file_path = self.absolute_dir_path + "/3d_shape/3d_shape.ftv"
        else:
            file_path = self.absolute_dir_path + "/3d_shape/3d_shape.bmp"

        return file_path

    # =============================================================================================
    # Function Name   :change_unicode_str
    # Description     :This function changes from unicode to str.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def change_unicode_str(self, string_val):
        if isinstance(string_val, unicode) is True:
            # unicode -> str
            return_value = string_val.encode("utf-8")
        else:
            return_value = string_val

        return return_value

    # =============================================================================================
    # Function Name   :change_str_unicode
    # Description     :This function changes from str to unicode.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def change_str_unicode(self, string_val):
        if isinstance(string_val, str) is True:
            # str -> unicode
            return_value = string_val.decode("utf-8")
        else:
            return_value = string_val

        return return_value

    # =============================================================================================
    # Function Name   :check_of_sys_mode
    # Description     :This function returns the system mode status.
    # Argument        :system mode
    # Return          :nothing special
    # =============================================================================================
    def check_of_sys_mode(self, mode):
        return_value = False

        if self.sys_mode == mode:
            return_value = True

        return return_value

    # =============================================================================================
    # Function Name   :check_of_play_mode
    # Description     :This function returns the play/stop status.
    # Argument        :system mode
    # Return          :nothing special
    # =============================================================================================
    def check_of_play_mode(self, mode):
        return_value = False

        if self.ply_mode == mode:
            return_value = True

        return return_value


# =================================================================================================
# =================================================================================================
class MyGuiForm(QtGui.QMainWindow):

    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        # GUI initial setting
        self.ui.tabWidget.setCurrentIndex(WINDOW_TAB__CAM)
        self.ui.r_btn_camera.setChecked(DF_DEFAULT_1SET)
        self.ui.r_btn_stop.setChecked(DF_DEFAULT_1SET)
        self.ui.Line_Edit_Z_adjust.setText(DF_DEFAULT_1CHR)
        self.ui.r_btn_wire_frame_off.setChecked(DF_DEFAULT_1SET)

        # Definition of variables
        self._source = None
        self.filters = "Image files \
            (*.bmp *.dib *.jpe *.jpeg *.jpg *.png *.tif *.tiff)"

        # GUI connection setting
        # ====================================================================
        QtCore.QObject.connect(self.ui.r_btn_camera,
                               QtCore.SIGNAL("clicked()"),
                               self.r_btn_camera_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.r_btn_capture,
                               QtCore.SIGNAL("clicked()"),
                               self.r_btn_capture_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.r_btn_cloud,
                               QtCore.SIGNAL("clicked()"),
                               self.r_btn_cloud_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.r_btn_stored_data,
                               QtCore.SIGNAL("clicked()"),
                               self.r_btn_stored_data_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.r_btn_play,
                               QtCore.SIGNAL("clicked()"),
                               self.r_btn_play_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.r_btn_stop,
                               QtCore.SIGNAL("clicked()"),
                               self.r_btn_stop_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_save_object,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_save_object_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.r_btn_wire_frame_off,
                               QtCore.SIGNAL("clicked()"),
                               self.r_btn_wire_frame_off_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.r_btn_wire_frame_on,
                               QtCore.SIGNAL("clicked()"),
                               self.r_btn_wire_frame_on_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.horizontalSlider,
                               QtCore.SIGNAL("valueChanged(int)"),
                               self.z_coordinate_slider_func)

        # tab capture frame
        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_opn_browse_left,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_opn_browse_left_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_chk_image_left,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_chk_image_left_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_opn_browse_center,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_opn_browse_center_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_chk_image_center,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_chk_image_center_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_opn_browse_right,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_opn_browse_right_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_chk_image_right,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_chk_image_right_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_select_complete,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_select_complete_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_path_inf_clear,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_path_inf_clear_func)

        # tab cloud-1 frame
        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_search_query_set,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_search_query_set_func)

        # tab cloud-2 frame
        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_file_select_left,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_file_select_left_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_file_select_center,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_file_select_center_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_file_select_right,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_file_select_right_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_chk_image_left_cloud,
                               QtCore.SIGNAL("clicked()"),
                               lambda: self.chk_image_cloud_f(KIND_F_NAME_LFT))

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_chk_image_center_cloud,
                               QtCore.SIGNAL("clicked()"),
                               lambda: self.chk_image_cloud_f(KIND_F_NAME_CEN))

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_chk_image_right_cloud,
                               QtCore.SIGNAL("clicked()"),
                               lambda: self.chk_image_cloud_f(KIND_F_NAME_RHT))

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_select_complete_cloud,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_select_complete_cloud_func)

        # ====================================================================
        QtCore.QObject.connect(self.ui.p_btn_file_inf_clear,
                               QtCore.SIGNAL("clicked()"),
                               self.p_btn_file_inf_clear_func)

    # Definition of GUI function
    # Main window function
    # =============================================================================================
    # Function Name   :r_btn_camera_func
    # Description     :The r_btn_camera_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def r_btn_camera_func(self):
        if self.validity_check_of_gui() is True:
            self.ui.tabWidget.setCurrentIndex(WINDOW_TAB__CAM)
            self._source.gui_camera_func()
        else:
            print messege_error_01

    # =============================================================================================
    # Function Name   :r_btn_capture_func
    # Description     :The r_btn_capture_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def r_btn_capture_func(self):
        if self.validity_check_of_gui() is True:
            self.ui.tabWidget.setCurrentIndex(WINDOW_TAB__CAP)
            self._source.gui_capture_func()
        else:
            print messege_error_01
            self.ui.r_btn_camera.setChecked(DF_DEFAULT_1SET)

    # =============================================================================================
    # Function Name   :r_btn_cloud_func
    # Description     :The r_btn_cloud_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def r_btn_cloud_func(self):
        if self.validity_check_of_gui() is True:
            if self.validity_check_of_db_connect() is True:
                self.ui.tabWidget.setCurrentIndex(WINDOW_TAB_CLD1)
                self._source.gui_cloud_func()
            else:
                QtGui.QMessageBox.warning(
                    self, "Message",
                    u"Database connection cannot be checked. \
Please check the connection of the database.")

        else:
            print messege_error_01
            self.ui.r_btn_camera.setChecked(DF_DEFAULT_1SET)

    # =============================================================================================
    # Function Name   :r_btn_stored_data_func
    # Description     :The r_btn_stored_data_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def r_btn_stored_data_func(self):
        if self.validity_check_of_gui() is True:
            self.ui.tabWidget.setCurrentIndex(WINDOW_TAB__STD)
            self._source.gui_stored_data_func()

            qtFilePath = QtGui.QFileDialog.getOpenFileName(
                self, 'Find object file', filter="*.ftv")

            if not qtFilePath.isNull():
                uiFilePath = copy.deepcopy(unicode(qtFilePath))
                if self.check_japan_string(uiFilePath) is True:
                    self._source.gui_stored_data_file_path_func(uiFilePath)
                else:
                    print messege_error_02
        else:
            print messege_error_01
            self.ui.r_btn_camera.setChecked(DF_DEFAULT_1SET)

    # =============================================================================================
    # Function Name   :r_btn_play_func
    # Description     :The r_btn_play_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def r_btn_play_func(self):
        if self.validity_check_of_gui() is True:
            self._source.gui_play_mode_func(DEFINE_MODE_PLY)
        else:
            print messege_error_01
            self.ui.r_btn_stop.setChecked(DF_DEFAULT_1SET)

    # =============================================================================================
    # Function Name   :r_btn_stop_func
    # Description     :The r_btn_stop_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def r_btn_stop_func(self):
        if self.validity_check_of_gui() is True:
            self._source.gui_play_mode_func(DEFINE_MODE_STP)
        else:
            print messege_error_01

    # =============================================================================================
    # Function Name   :p_btn_save_object_func
    # Description     :The p_btn_save_object_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_save_object_func(self):
        if self.validity_check_of_gui() is True:
            savename = QtGui.QFileDialog.getSaveFileName(
                self, 'Save object file')
            if not savename.isNull():
                file_name_path = copy.deepcopy(unicode(savename))
                if self.check_japan_string(file_name_path) is True:
                    self.p_btn_save_object_sub_func(file_name_path)
                else:
                    print messege_error_02
        else:
            print messege_error_01

    # =============================================================================================
    # Function Name   :p_btn_save_object_sub_func
    # Description     :This function called from p_btn_save_object_func.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_save_object_sub_func(self, file_name_path):
        try:
            file_name = os.path.basename(file_name_path)
        except Exception as err:
            print err.message
        else:
            f_name, ext = os.path.splitext(file_name)
            ext = ext.lower()
            if ext == ".ftv":
                if len(f_name) < NUM_OF_CHAR_MAX:
                    chk_name = f_name.lower()
                    if chk_name != "3d_shape":
                        dir_name = os.path.dirname(file_name_path)
                        self._source.gui_stored_data_file_name_func(
                            f_name, dir_name)
                    else:
                        QtGui.QMessageBox.warning(
                            self, "Message",
                            u"Please make it a different file name")
                else:
                    QtGui.QMessageBox.warning(
                        self, "Message",
                        u"File name length is up to 45 characters")
            else:
                QtGui.QMessageBox.warning(
                    self, "Message",
                    u"Please file extension ftv")

    # =============================================================================================
    # Function Name   :r_btn_wire_frame_off_func
    # Description     :The r_btn_wire_frame_off_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def r_btn_wire_frame_off_func(self):
        if self.validity_check_of_gui() is True:
            self._source.gui_wire_frame_mode_func(W_FRAME_MOD_OFF)
        else:
            print messege_error_01

    # =============================================================================================
    # Function Name   :r_btn_wire_frame_on_func
    # Description     :The r_btn_wire_frame_on_func function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def r_btn_wire_frame_on_func(self):
        if self.validity_check_of_gui() is True:
            self._source.gui_wire_frame_mode_func(W_FRAME_MOD__ON)
        else:
            print messege_error_01
            self.ui.r_btn_wire_frame_off.setChecked(DF_DEFAULT_1SET)

    # =============================================================================================
    # Function Name   :z_coordinate_slider_func
    # Description     :This function gets the value of the slider.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def z_coordinate_slider_func(self):
        if self.validity_check_of_gui() is True:
            if self.ui.horizontalSlider.value() == DF_DEFAULT_0SET:
                self.ui.Line_Edit_Z_adjust.setText(DF_DEFAULT_1CHR)
                self.ui.horizontalSlider.setValue(DF_DEFAULT_1SET)
            else:
                self.ui.Line_Edit_Z_adjust.setText(
                    str(self.ui.horizontalSlider.value()))

            qt_slider_val = self.ui.horizontalSlider.value()
            slider_val = copy.deepcopy(unicode(qt_slider_val))
            self._source.gui_z_coordinate_slider_func(slider_val)
        else:
            print messege_error_01
            self.ui.Line_Edit_Z_adjust.setText(DF_DEFAULT_1CHR)
            self.ui.horizontalSlider.setValue(DF_DEFAULT_1SET)

    # =============================================================================================
    # Function Name   :p_btn_opn_browse_left_func
    # Description     :The p_btn_opn_browse_left_func function called from GUI
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_opn_browse_left_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CAP) is True:
            qt_file_path = QtGui.QFileDialog.getOpenFileName(
                self, filter=self.filters)

            self.check_file_path_and_set(qt_file_path, KIND_F_PATH_LFT)

    # =============================================================================================
    # Function Name   :p_btn_chk_image_left_func
    # Description     :The p_btn_chk_image_left_func function called from GUI
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_chk_image_left_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CAP) is True:
            qt_file_path = self.ui.Line_Edit_path_img_left.text()
            if not qt_file_path.isNull():
                file_path = copy.deepcopy(unicode(qt_file_path))
                self._source.gui_chk_image(file_path)

    # =============================================================================================
    # Function Name   :p_btn_opn_browse_center_func
    # Description     :The p_btn_opn_browse_center_func
    #                 :function called from GUI
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_opn_browse_center_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CAP) is True:
            qt_file_path = QtGui.QFileDialog.getOpenFileName(
                self, filter=self.filters)

            self.check_file_path_and_set(qt_file_path, KIND_F_PATH_CEN)

    # =============================================================================================
    # Function Name   :p_btn_chk_image_center_func
    # Description     :The p_btn_chk_image_center_func function called from GUI
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_chk_image_center_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CAP) is True:
            qt_file_path = self.ui.Line_Edit_path_img_center.text()
            if not qt_file_path.isNull():
                file_path = copy.deepcopy(unicode(qt_file_path))
                self._source.gui_chk_image(file_path)

    # =============================================================================================
    # Function Name   :p_btn_opn_browse_right_func
    # Description     :The p_btn_opn_browse_right_func function called from GUI
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_opn_browse_right_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CAP) is True:
            qt_file_path = QtGui.QFileDialog.getOpenFileName(
                self, filter=self.filters)

            self.check_file_path_and_set(qt_file_path, KIND_F_PATH_RHT)

    # =============================================================================================
    # Function Name   :p_btn_chk_image_right_func
    # Description     :The p_btn_chk_image_right_func function called from GUI
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_chk_image_right_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CAP) is True:
            qt_file_path = self.ui.Line_Edit_path_img_right.text()
            if not qt_file_path.isNull():
                file_path = copy.deepcopy(unicode(qt_file_path))
                self._source.gui_chk_image(file_path)

    # =============================================================================================
    # Function Name   :p_btn_select_complete_func
    # Description     :The p_btn_select_complete_func function called from GUI
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_select_complete_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CAP) is True:
            self._source.gui_select_complete_func()

    # =============================================================================================
    # Function Name   :p_btn_path_inf_clear_func
    # Description     :The p_btn_path_inf_clear_func function called from GUI
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_path_inf_clear_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CAP) is True:
            self.ui.Line_Edit_path_img_left.clear()
            self.ui.Line_Edit_path_img_center.clear()
            self.ui.Line_Edit_path_img_right.clear()
            self._source.gui_path_inf_clear_func()

    # =============================================================================================
    # Function Name   :p_btn_search_query_set_func
    # Description     :The p_btn_search_query_set_func
    #                 :function called from GUI
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_search_query_set_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CLD) is True:
            result_robot_id = True
            result_date_tim = False
            qt_robot_id = self.ui.Line_Edit_robot_id.text()
            if not qt_robot_id.isNull():
                robot_id = copy.deepcopy(unicode(qt_robot_id))
                if self.check_robot_id(robot_id):
                    qt_y_min = self.ui.Line_Edit_date_years.text()
                    qt_t_min = self.ui.Line_Edit_date_month.text()
                    qt_d_min = self.ui.Line_Edit_date_day.text()
                    qt_y_max = self.ui.Line_Edit_date_years_max.text()
                    qt_t_max = self.ui.Line_Edit_date_month_max.text()
                    qt_d_max = self.ui.Line_Edit_date_day_max.text()
                    if self.check_null_date_date(qt_y_min, qt_t_min, qt_d_min):
                        if self.check_null_date_date(
                                qt_y_max, qt_t_max, qt_d_max):

                            qt_h_min = self.ui.Line_Edit_date_l_hour.text()
                            qt_m_min = self.ui.Line_Edit_date_l_minute.text()
                            qt_s_min = self.ui.Line_Edit_date_l_second.text()
                            qt_h_max = self.ui.Line_Edit_date_u_hour.text()
                            qt_m_max = self.ui.Line_Edit_date_u_minute.text()
                            qt_s_max = self.ui.Line_Edit_date_u_second.text()
                            if self.check_null_date_time(
                                   qt_h_min, qt_m_min, qt_s_min):
                                if self.check_null_date_time(
                                       qt_h_max, qt_m_max, qt_s_max):

                                    year_min = copy.deepcopy(unicode(qt_y_min))
                                    mon__min = copy.deepcopy(unicode(qt_t_min))
                                    day__min = copy.deepcopy(unicode(qt_d_min))
                                    year_max = copy.deepcopy(unicode(qt_y_max))
                                    mon__max = copy.deepcopy(unicode(qt_t_max))
                                    day__max = copy.deepcopy(unicode(qt_d_max))

                                    h_min = copy.deepcopy(unicode(qt_h_min))
                                    m_min = copy.deepcopy(unicode(qt_m_min))
                                    s_min = copy.deepcopy(unicode(qt_s_min))
                                    h_max = copy.deepcopy(unicode(qt_h_max))
                                    m_max = copy.deepcopy(unicode(qt_m_max))
                                    s_max = copy.deepcopy(unicode(qt_s_max))
                                if self.validity_check_of_date(
                                       year_min, mon__min, day__min,
                                       year_max, mon__max, day__max,
                                       h_min, m_min, s_min,
                                       h_max, m_max, s_max):
                                    self._source.gui_outputs_db_search_query(
                                        robot_id, year_min, mon__min, day__min,
                                        year_max, mon__max, day__max, h_min,
                                        m_min, s_min, h_max, m_max, s_max)
                                    self.ui.tabWidget.setCurrentIndex(
                                        WINDOW_TAB_CLD2)
                                    result_date_tim = True
                else:
                    result_robot_id = False
            else:
                result_robot_id = False

            if result_robot_id is False:
                QtGui.QMessageBox.warning(
                    self, "Message", u"Robot ID is wrong. Please confirm.")
            else:
                if result_date_tim is False:
                    QtGui.QMessageBox.warning(
                        self, "Message",
                        u"Date data is wrong. Please confirm.")

    # =============================================================================================
    # Function Name   :check_null_date_date
    # Description     :This function confirms the validity of the date.
    # Argument        :date data
    # Return          :information of normal end(True) or abnormal end(False).
    # =============================================================================================
    def check_null_date_date(self, qt_year, qt__mon, qt__day):
        if qt_year.isNull():
            return False
        if qt__mon.isNull():
            return False
        if qt__day.isNull():
            return False

        return True

    # =============================================================================================
    # Function Name   :check_null_date_time
    # Description     :This function confirms the validity of the date time.
    # Argument        :date time
    # Return          :information of normal end(True) or abnormal end(False).
    # =============================================================================================
    def check_null_date_time(self, qt_hour, qt_min, qt_sec):
        if qt_hour.isNull():
            return False
        if qt_min.isNull():
            return False
        if qt_sec.isNull():
            return False

        return True

    # =============================================================================================
    # Function Name   :check_robot_id
    # Description     :This function confirms the validity of the robot id.
    # Argument        :robot id data
    # Return          :information of normal end(True) or abnormal end(False).
    # =============================================================================================
    def check_robot_id(self, bobot_id):
        if bobot_id == "drone1":
            return True
        elif bobot_id == "drone2":
            return True
        elif bobot_id == "drone3":
            return True
        elif bobot_id == "drone4":
            return True
        elif bobot_id == "drone5":
            return True
        elif bobot_id == "drone6":
            return True
        elif bobot_id == "spider1":
            return True
        elif bobot_id == "spider2":
            return True
        elif bobot_id == "spider3":
            return True
        elif bobot_id == "enryu1":
            return True
        elif bobot_id == "enryu2":
            return True
        elif bobot_id == "enryu3":
            return True
        elif bobot_id == "gorone1":
            return True
        elif bobot_id == "gorone2":
            return True
        elif bobot_id == "gorone3":
            return True
        elif bobot_id == "gorone4":
            return True
        elif bobot_id == "gorone5":
            return True
        elif bobot_id == "gorone6":
            return True
        else:
            return False

    # =============================================================================================
    # Function Name   :validity_check_of_date
    # Description     :This function confirms the validity of the date.
    # Argument        :date data
    # Return          :information of normal end(True) or abnormal end(False).
    # =============================================================================================
    def validity_check_of_date(
            self, min_year, min_mon, min_day, max_year, max_mon, max_day,
            h_min, m_min, s_min, h_max, m_max, s_max):

        result = False

        date_time_min = min_year + "/" + min_mon + "/" + min_day \
            + " " + h_min + ":" + m_min + ":" + s_min
        date_time_max = max_year + "/" + max_mon + "/" + max_day \
            + " " + h_max + ":" + m_max + ":" + s_max

        try:
            date_time = dateutil.parser.parse(date_time_min)
        except Exception as err:
            print err.message
        else:
            try:
                date_time = dateutil.parser.parse(date_time_max)
            except Exception as err:
                print err.message
            else:
                year_min = copy.deepcopy(int(min_year))
                mon__min = copy.deepcopy(int(min_mon))
                day__min = copy.deepcopy(int(min_day))
                hour_min = copy.deepcopy(int(h_min))
                min__min = copy.deepcopy(int(m_min))
                sec__min = copy.deepcopy(int(s_min))

                year_max = copy.deepcopy(int(max_year))
                mon__max = copy.deepcopy(int(max_mon))
                day__max = copy.deepcopy(int(max_day))
                hour_max = copy.deepcopy(int(h_max))
                min__max = copy.deepcopy(int(m_max))
                sec__max = copy.deepcopy(int(s_max))

                date_min = datetime.date(
                    year_min, mon__min, day__min)
                date_max = datetime.date(
                    year_max, mon__max, day__max)

                datetime_min = datetime.time(
                    hour_min, min__min, sec__min)
                datetime_max = datetime.time(
                    hour_max, min__max, sec__max)

                if date_max >= date_min:
                    if date_max == date_min:
                        if datetime_max > datetime_min:
                            result = True
                    else:
                        result = True

        return result

    # =============================================================================================
    # Function Name   :p_btn_file_select_left_func
    # Description     :The p_btn_file_select_left_func
    #                 :function called from GUI.
    # Argument        :image-name list
    # Return          :nothing special
    # =============================================================================================
    def p_btn_file_select_left_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CLD) is True:
            self.popup_sub_window(self._source.name_list, KIND_F_NAME_LFT)

    # =============================================================================================
    # Function Name   :p_btn_file_select_center_func
    # Description     :The p_btn_file_select_center_func
    #                 :function called from GUI.
    # Argument        :image-name list
    # Return          :nothing special
    # =============================================================================================
    def p_btn_file_select_center_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CLD) is True:
            self.popup_sub_window(self._source.name_list, KIND_F_NAME_CEN)

    # =============================================================================================
    # Function Name   :p_btn_file_select_right_func
    # Description     :The p_btn_file_select_right_func
    #                 :function called from GUI.
    # Argument        :image-name list
    # Return          :nothing special
    # =============================================================================================
    def p_btn_file_select_right_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CLD) is True:
            self.popup_sub_window(self._source.name_list, KIND_F_NAME_RHT)

    # =============================================================================================
    # Function Name   :popup_sub_window
    # Description     :This function display a window that gets list data.
    # Argument        :image-name list
    # Return          :nothing special
    # =============================================================================================
    def popup_sub_window(self, list_data, kind):
        if len(list_data) > 0:
            self.sub_ui = MySubDialog(self)

            QtCore.QObject.connect(self.sub_ui.sub_window_set,
                                   QtCore.SIGNAL("clicked()"),
                                   lambda: self.p_btn_sub_file_select_set_func
                                   (kind))

            self.sub_ui.listWidget.clear()
            for data in list_data:
                self.sub_ui.listWidget.addItems([data])

            self.sub_ui.listWidget.itemDoubleClicked.connect(
                self.on_DoubleClicked)
            self.sub_ui.setFixedSize(WINDOW_S_SIZE_W, WINDOW_S_SIZE_H)
            self.sub_ui.setWindowTitle("File selection")
            self.sub_ui.lineEdit.clear()
            self.sub_ui.show()

        else:
            QtGui.QMessageBox.warning(
                self, "Message",
                u"List information has not been prepared yet. \
Please reset the search condition again.")

    # =============================================================================================
    # Function Name   :p_btn_sub_file_select_set_func
    # Description     :This function copies the selected image-file name
    #                 :to the main window.
    # Argument        :image-file name
    # Return          :nothing special
    # =============================================================================================
    def p_btn_sub_file_select_set_func(self, kind):
        result = True
        if kind == KIND_F_NAME_LFT:
            self.ui.Line_Edit_file_name_left.setText(
                self.sub_ui.lineEdit.text())
        elif kind == KIND_F_NAME_CEN:
            self.ui.Line_Edit_file_name_center.setText(
                self.sub_ui.lineEdit.text())
        elif kind == KIND_F_NAME_RHT:
            self.ui.Line_Edit_file_name_right.setText(
                self.sub_ui.lineEdit.text())
        else:
            result = False

        if result is True:
            qt_f_name = self.sub_ui.lineEdit.text()
            file_name = copy.deepcopy(unicode(qt_f_name))
            self._source.gui_set_file_name(file_name, kind)
            self.sub_ui.close()

    # =============================================================================================
    # Function Name   :on_DoubleClicked
    # Description     :This function obtains the file name
    #                 :by double clicking the mouse.
    # Argument        :image-file name list
    # Return          :nothing special
    # =============================================================================================
    def on_DoubleClicked(self):
        currentListIndex = self.sub_ui.listWidget.currentRow()

        if currentListIndex != (-1):
            self.sub_ui.lineEdit.setText(
                self.sub_ui.listWidget.currentItem().text())

    # =============================================================================================
    # Function Name   :chk_image_cloud_f
    # Description     :The chk_image_cloud_f function called from GUI.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def chk_image_cloud_f(self, kind):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CLD) is True:

            result = True

            if kind == KIND_F_NAME_LFT:
                qt_file_name = self.ui.Line_Edit_file_name_left.text()
            elif kind == KIND_F_NAME_CEN:
                qt_file_name = self.ui.Line_Edit_file_name_center.text()
            elif kind == KIND_F_NAME_RHT:
                qt_file_name = self.ui.Line_Edit_file_name_right.text()
            else:
                result = False

            if result is True:
                if not qt_file_name.isNull():
                    file_name = copy.deepcopy(unicode(qt_file_name))
                    self._source.gui_chk_image_cloud(file_name)

    # =============================================================================================
    # Function Name   :p_btn_select_complete_cloud_func
    # Description     :The p_btn_select_complete_cloud_func
    #                 :function called from GUI
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_select_complete_cloud_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CLD) is True:
            self._source.gui_select_complete_cloud_func()

    # =============================================================================================
    # Function Name   :set_data_source
    # Description     :This function sets an instance of
    #                 :     RTC class to "self._source".
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def set_data_source(self, source):
        self._source = source
        return

    # =============================================================================================
    # Function Name   :validity_check_of_gui
    # Description     :This function returns the status of RTC activation.
    # Argument        :nothing special
    # Return          :RTC activation status
    # =============================================================================================
    def validity_check_of_gui(self):
        return_value = False

        if self._source.activated_sts == DF_ON_ACTIVATED:
            return_value = True

        return return_value

    # =============================================================================================
    # Function Name   :validity_check_of_sys_mode
    # Description     :This function returns the system mode status.
    # Argument        :system mode
    # Return          :nothing special
    # =============================================================================================
    def validity_check_of_sys_mode(self, mode):
        return_value = False

        if self._source.sys_mode == mode:
            return_value = True

        return return_value

    # =============================================================================================
    # Function Name   :validity_check_of_db_connect
    # Description     :This function returns the DB connection status.
    # Argument        :DB connection status
    # Return          :nothing special
    # =============================================================================================
    def validity_check_of_db_connect(self):
        return_value = False

        if self._source.db_connect_sts == DB_CNECT_NUM_OK:
            return_value = True

        return return_value

    # =============================================================================================
    # Function Name   :check_japan_string
    # Description     :This function checks whether
    #                 :     the string contains Japanese.
    # Argument        :string
    # Return          :information of normal end or abnormal end.
    # =============================================================================================
    def check_japan_string(self, value):
        for chr in value:
            ret = unicodedata.east_asian_width(chr) in ['F', 'H', 'W', 'A']
            if ret is True:
                return False

        return True

    # =============================================================================================
    # Function Name   :check_file_exists
    # Description     :This function checks whether a file exists.
    # Argument        :file path
    # Return          :information of normal end or abnormal end.
    # =============================================================================================
    def check_file_exists(self, value):
        if os.path.exists(value) is False:
            return False
        else:
            if os.path.isfile(value) is False:
                return False

        return True

    # =============================================================================================
    # Function Name   :check_file_path_and_set
    # Description     :This function sets the file path information
    #                 :    of the image.
    # Argument        :file path information
    # Return          :nothing special
    # =============================================================================================
    def check_file_path_and_set(self, qt_file_path, kind_val):
        if not qt_file_path.isNull():
            file_path = copy.deepcopy(unicode(qt_file_path))
            if self.check_japan_string(file_path):
                if kind_val == KIND_F_PATH_LFT:
                    self.ui.Line_Edit_path_img_left.setText(file_path)
                    self._source.gui_set_file_path(KIND_F_PATH_LFT, file_path)
                elif kind_val == KIND_F_PATH_CEN:
                    self.ui.Line_Edit_path_img_center.setText(file_path)
                    self._source.gui_set_file_path(KIND_F_PATH_CEN, file_path)
                elif kind_val == KIND_F_PATH_RHT:
                    self.ui.Line_Edit_path_img_right.setText(file_path)
                    self._source.gui_set_file_path(KIND_F_PATH_RHT, file_path)
                else:
                    pass
            else:
                print messege_error_02

    # =============================================================================================
    # Function Name   :p_btn_file_inf_clear_func
    # Description     :The p_btn_file_inf_clear_func function called from GUI
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def p_btn_file_inf_clear_func(self):
        if self.validity_check_of_sys_mode(DEFINE_MODE_CLD) is True:
            self.ui.Line_Edit_file_name_left.clear()
            self.ui.Line_Edit_file_name_center.clear()
            self.ui.Line_Edit_file_name_right.clear()
            self._source.gui_file_inf_clear_func()

    # =============================================================================================
    # Function Name   :global_gui_initialize
    # Description     :This function sets screen of GUI to the initial state.
    # Argument        :nothing special
    # Return          :nothing special
    # =============================================================================================
    def global_gui_initialize(self):
        self.ui.r_btn_camera.setChecked(DF_DEFAULT_1SET)
        self.ui.r_btn_stop.setChecked(DF_DEFAULT_1SET)
        self.ui.r_btn_wire_frame_off.setChecked(DF_DEFAULT_1SET)
        self.ui.Line_Edit_Z_adjust.setText(DF_DEFAULT_1CHR)
        self.ui.horizontalSlider.setValue(DF_DEFAULT_1SET)

        self.ui.tabWidget.setCurrentIndex(WINDOW_TAB__CAM)

        self.ui.Line_Edit_path_img_left.clear()
        self.ui.Line_Edit_path_img_center.clear()
        self.ui.Line_Edit_path_img_right.clear()

        self.ui.Line_Edit_robot_id.clear()

        self.ui.Line_Edit_date_years.clear()
        self.ui.Line_Edit_date_month.clear()
        self.ui.Line_Edit_date_day.clear()

        self.ui.Line_Edit_date_l_hour.clear()
        self.ui.Line_Edit_date_l_minute.clear()
        self.ui.Line_Edit_date_l_second.clear()

        self.ui.Line_Edit_date_years_max.clear()
        self.ui.Line_Edit_date_month_max.clear()
        self.ui.Line_Edit_date_day_max.clear()

        self.ui.Line_Edit_date_u_hour.clear()
        self.ui.Line_Edit_date_u_minute.clear()
        self.ui.Line_Edit_date_u_second.clear()

        self.ui.Line_Edit_file_name_left.clear()
        self.ui.Line_Edit_file_name_center.clear()
        self.ui.Line_Edit_file_name_right.clear()

# =================================================================================================
# =================================================================================================


class MySubDialog(QtGui.QDialog, Ui_Dialog):

    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.activateManager()

    profile = OpenRTM_aist.Properties(defaults_str=rtc_3dimageshapegui_spec)
    mgr.registerFactory(profile,
                        rtc_3DImageShapeGUI,
                        OpenRTM_aist.Delete)

    # Create a component
    comp = mgr.createComponent("rtc_3DImageShapeGUI")

    mgr.runManager(True)

    app = QtGui.QApplication(sys.argv)
    global my_gui_form
    my_gui_form = MyGuiForm()
    my_gui_form.set_data_source(comp)
    my_gui_form.setFixedSize(WINDOW_M_SIZE_W, WINDOW_M_SIZE_H)
    my_gui_form.setWindowTitle("3D image shape system GUI")
    my_gui_form.show()
    sys.exit(app.exec_())

    mgr.shutdown()

if __name__ == "__main__":
    main()
